import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// Create a `StatelessWidget` using `ChangeNotifierProvider`.
class StatelessMutablePVC<T extends ChangeNotifier> extends StatelessWidget {
  /// The controller for this view.
  ///
  /// Must implement `ChangeNotifier`, and handle the state of the view by calling `notifyListeners`.
  final T controller;

  /// The widget returned by the `builder` function rebuild itself whenever the `controller` changes its state by calling `notifyListeners`.
  ///
  ///  The `child` widget is used for optimization. The `child` widget won't rebuild itself when the `controller` changes its state.
  final Widget Function(BuildContext context, T model, Widget child) builder;

  /// Passed to the `builder` function. It won't rebuild itself when the `controller` changes its state.
  final Widget child;

  /// Create a `StatelessWidget` using `ChangeNotifierProvider`, and bind the `controller` to it.
  ///
  /// `StatelessMutablePVC` rebuild itself whenever the `controller` call `notifyListeners`.
  StatelessMutablePVC({
    @required this.controller,
    @required this.builder,
    this.child,
  });

  @override
  Widget build(BuildContext context) {
    return ChangeNotifierProvider<T>(
      builder: (context) => controller,
      child: Consumer<T>(builder: builder),
    );
  }
}
